/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#include "Heavy_WSTD_FLDR.hpp"

#include <new>

#define Context(_c) static_cast<Heavy_WSTD_FLDR *>(_c)


/*
 * C Functions
 */

extern "C" {
  HV_EXPORT HeavyContextInterface *hv_WSTD_FLDR_new(double sampleRate) {
    // allocate aligned memory
    void *ptr = hv_malloc(sizeof(Heavy_WSTD_FLDR));
    // ensure non-null
    if (!ptr) return nullptr;
    // call constructor
    new(ptr) Heavy_WSTD_FLDR(sampleRate);
    return Context(ptr);
  }

  HV_EXPORT HeavyContextInterface *hv_WSTD_FLDR_new_with_options(double sampleRate,
      int poolKb, int inQueueKb, int outQueueKb) {
    // allocate aligned memory
    void *ptr = hv_malloc(sizeof(Heavy_WSTD_FLDR));
    // ensure non-null
    if (!ptr) return nullptr;
    // call constructor
    new(ptr) Heavy_WSTD_FLDR(sampleRate, poolKb, inQueueKb, outQueueKb);
    return Context(ptr);
  }

  HV_EXPORT void hv_WSTD_FLDR_free(HeavyContextInterface *instance) {
    // call destructor
    Context(instance)->~Heavy_WSTD_FLDR();
    // free memory
    hv_free(instance);
  }
} // extern "C"







/*
 * Class Functions
 */

Heavy_WSTD_FLDR::Heavy_WSTD_FLDR(double sampleRate, int poolKb, int inQueueKb, int outQueueKb)
    : HeavyContext(sampleRate, poolKb, inQueueKb, outQueueKb) {
  numBytes += sLine_init(&sLine_EapmnhjA);
  numBytes += sEnv_init(&sEnv_I3YEyvOa, 512, 256);
  numBytes += sEnv_init(&sEnv_vY4kIDob, 512, 256);
  numBytes += sEnv_init(&sEnv_HZ3RTB6V, 512, 256);
  numBytes += sEnv_init(&sEnv_OAnIN3rw, 512, 256);
  numBytes += sLine_init(&sLine_9izQKvwX);
  numBytes += sLine_init(&sLine_TNcFXvPg);
  numBytes += cVar_init_f(&cVar_x0rXjgh0, 0.0f);
  numBytes += cVar_init_f(&cVar_ShFnw41F, 0.0f);
  numBytes += sVarf_init(&sVarf_tFRdCFny, 0.0f, 0.0f, false);
  numBytes += cVar_init_f(&cVar_kEC9XyTZ, 0.0f);
  numBytes += cIf_init(&cIf_gmxU13Fs, false);
  numBytes += cIf_init(&cIf_j1IshWSn, false);
  numBytes += cBinop_init(&cBinop_pMIzpqZH, 0.0f); // __sub
  numBytes += sVarf_init(&sVarf_08mScSWn, 0.0f, 0.0f, false);
  numBytes += cVar_init_f(&cVar_RXW5loWX, 0.0f);
  numBytes += cIf_init(&cIf_TCycJSp0, false);
  numBytes += cIf_init(&cIf_SGyujIhK, false);
  numBytes += cBinop_init(&cBinop_8XV3yZLt, 0.0f); // __sub
  numBytes += cVar_init_f(&cVar_S0dzBDYR, 1.0f);
  numBytes += sVarf_init(&sVarf_H1kktSyo, 0.0f, 0.0f, false);
  numBytes += sVarf_init(&sVarf_3yolUEdJ, 0.0f, 0.0f, false);
  numBytes += sVarf_init(&sVarf_sxxkYU0s, 0.0f, 0.0f, false);
  numBytes += sVarf_init(&sVarf_UqUr6y8w, 0.0f, 0.0f, false);
  
  // schedule a message to trigger all loadbangs via the __hv_init receiver
  scheduleMessageForReceiver(0xCE5CC65B, msg_initWithBang(HV_MESSAGE_ON_STACK(1), 0));
}

Heavy_WSTD_FLDR::~Heavy_WSTD_FLDR() {
  sEnv_free(&sEnv_I3YEyvOa);
  sEnv_free(&sEnv_vY4kIDob);
  sEnv_free(&sEnv_HZ3RTB6V);
  sEnv_free(&sEnv_OAnIN3rw);
}

HvTable *Heavy_WSTD_FLDR::getTableForHash(hv_uint32_t tableHash) {
  return nullptr;
}

void Heavy_WSTD_FLDR::scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) {
  switch (receiverHash) {
    case 0xE42C892E: { // Fldr
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_GQRxiRAj_sendMessage);
      break;
    }
    case 0xCB23DC60: { // Lmtr
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_LCQlUSXJ_sendMessage);
      break;
    }
    case 0x97D46AC6: { // Mix
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_KRkbElfJ_sendMessage);
      break;
    }
    case 0xCF8296B9: { // 1001-mix
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_Cl4lHlXR_sendMessage);
      break;
    }
    case 0xCE5CC65B: { // __hv_init
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_ihp3waXU_sendMessage);
      break;
    }
    default: return;
  }
}

int Heavy_WSTD_FLDR::getParameterInfo(int index, HvParameterInfo *info) {
  if (info != nullptr) {
    switch (index) {
      case 0: {
        info->name = "Fldr";
        info->hash = 0xE42C892E;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 1.0f;
        info->maxVal = 13.37f;
        info->defaultVal = 1.0f;
        break;
      }
      case 1: {
        info->name = "Lmtr";
        info->hash = 0xCB23DC60;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 0.0f;
        info->maxVal = 1.0f;
        info->defaultVal = 1.0f;
        break;
      }
      case 2: {
        info->name = "Mix";
        info->hash = 0x97D46AC6;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 0.0f;
        info->maxVal = 100.0f;
        info->defaultVal = 50.0f;
        break;
      }
      default: {
        info->name = "invalid parameter index";
        info->hash = 0;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 0.0f;
        info->maxVal = 0.0f;
        info->defaultVal = 0.0f;
        break;
      }
    }
  }
  return 3;
}



/*
 * Send Function Implementations
 */


void Heavy_WSTD_FLDR::cBinop_0L9howgm_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MAX, 0.0f, 0, m, &cBinop_w17KrEW1_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_w17KrEW1_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MULTIPLY, 2.0f, 0, m, &cBinop_esfZ05Jc_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_esfZ05Jc_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 1.0f, 0, m, &cBinop_oHDU39sz_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_oHDU39sz_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_0uLefYhK_sendMessage);
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_PtMAqfZJ_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_PtMAqfZJ_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_IkpB58p1_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cCast_0uLefYhK_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_ADD, 1.0f, 0, m, &cBinop_o4ASLpzl_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_o4ASLpzl_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MULTIPLY, 0.5f, 0, m, &cBinop_o2iN37Zd_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_IkpB58p1_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setFloat(m, 0, 1.0f);
  msg_setElementToFrom(m, 1, n, 0);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 0.0f, 0, m, &cBinop_BMwxYGUV_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_BMwxYGUV_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MULTIPLY, 0.5f, 0, m, &cBinop_GlpWJDOc_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_o2iN37Zd_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_5BqkfS9f_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cBinop_GlpWJDOc_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_Oof5MhVd_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cVar_x0rXjgh0_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cSend_HUNEtBB3_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cCast_eoUG4Us9_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_vdxQ66Ft_sendMessage(_c, 0, m);
  cMsg_h4i2XD7l_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cVar_ShFnw41F_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_z3OOyjHA_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cVar_kEC9XyTZ_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_pMIzpqZH, HV_BINOP_SUBTRACT, 0, m, &cBinop_pMIzpqZH_sendMessage);
}

void Heavy_WSTD_FLDR::cIf_gmxU13Fs_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_tFRdCFny, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_FLDR::cIf_j1IshWSn_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_tFRdCFny, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_FLDR::sEnv_I3YEyvOa_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_kEC9XyTZ, 0, m, &cVar_kEC9XyTZ_sendMessage);
}

void Heavy_WSTD_FLDR::sEnv_vY4kIDob_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_rxxZNeFn_sendMessage);
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_EtZB5pnc_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_dgMDta5Z_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_pMIzpqZH_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_0U9xWbv8_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cMsg_0U9xWbv8_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  msg_setElementToFrom(m, 1, n, 0);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 0.0f, 0, m, &cBinop_Ykokv1Q5_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_Ykokv1Q5_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MIN, 100.0f, 0, m, &cBinop_o3pODnnw_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_ronlnmAF_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_kFE29kcD_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_W3YpdWjS_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_W3YpdWjS_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_Uirb6scD_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cCast_kFE29kcD_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_gmxU13Fs, 1, m, &cIf_gmxU13Fs_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_MQgTonZi_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_j1IshWSn, 1, m, &cIf_j1IshWSn_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_Uirb6scD_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  cIf_onMessage(_c, &Context(_c)->cIf_gmxU13Fs, 0, m, &cIf_gmxU13Fs_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_o3pODnnw_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_j1IshWSn, 0, m, &cIf_j1IshWSn_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_EtZB5pnc_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_pMIzpqZH, HV_BINOP_SUBTRACT, 1, m, &cBinop_pMIzpqZH_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_rxxZNeFn_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
}

void Heavy_WSTD_FLDR::cCast_dgMDta5Z_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_kEC9XyTZ, 0, m, &cVar_kEC9XyTZ_sendMessage);
}

void Heavy_WSTD_FLDR::cVar_RXW5loWX_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_8XV3yZLt, HV_BINOP_SUBTRACT, 0, m, &cBinop_8XV3yZLt_sendMessage);
}

void Heavy_WSTD_FLDR::cIf_TCycJSp0_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_08mScSWn, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_FLDR::cIf_SGyujIhK_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_08mScSWn, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_FLDR::sEnv_HZ3RTB6V_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_RXW5loWX, 0, m, &cVar_RXW5loWX_sendMessage);
}

void Heavy_WSTD_FLDR::sEnv_OAnIN3rw_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_7cgjUbNM_sendMessage);
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_XtDeUwUx_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_0nAsFLBA_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_8XV3yZLt_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_niQfnDWN_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cMsg_niQfnDWN_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  msg_setElementToFrom(m, 1, n, 0);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 0.0f, 0, m, &cBinop_ILWuA3Kj_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_ILWuA3Kj_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MIN, 100.0f, 0, m, &cBinop_fluVyqkI_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_GpdJq9Rv_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_H9qgHDE3_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_A7UxbEqk_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_A7UxbEqk_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_iYeJ7Rmf_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cCast_H9qgHDE3_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_TCycJSp0, 1, m, &cIf_TCycJSp0_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_eDdqYkOv_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_SGyujIhK, 1, m, &cIf_SGyujIhK_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_iYeJ7Rmf_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  cIf_onMessage(_c, &Context(_c)->cIf_TCycJSp0, 0, m, &cIf_TCycJSp0_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_fluVyqkI_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_SGyujIhK, 0, m, &cIf_SGyujIhK_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_XtDeUwUx_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_8XV3yZLt, HV_BINOP_SUBTRACT, 1, m, &cBinop_8XV3yZLt_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_0nAsFLBA_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_RXW5loWX, 0, m, &cVar_RXW5loWX_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_7cgjUbNM_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
}

void Heavy_WSTD_FLDR::cVar_S0dzBDYR_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 0.0f, 0, m, &cBinop_tuWzjzdS_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 0.0f, 0, m, &cBinop_ronlnmAF_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 1.0f, 0, m, &cBinop_MQgTonZi_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 0.0f, 0, m, &cBinop_GpdJq9Rv_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 1.0f, 0, m, &cBinop_eDdqYkOv_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_tuWzjzdS_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_S0dzBDYR, 1, m, &cVar_S0dzBDYR_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_5BqkfS9f_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setElementToFrom(m, 0, n, 0);
  msg_setFloat(m, 1, 100.0f);
  sLine_onMessage(_c, &Context(_c)->sLine_TNcFXvPg, 0, m, NULL);
}

void Heavy_WSTD_FLDR::cMsg_Oof5MhVd_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setElementToFrom(m, 0, n, 0);
  msg_setFloat(m, 1, 100.0f);
  sLine_onMessage(_c, &Context(_c)->sLine_9izQKvwX, 0, m, NULL);
}

void Heavy_WSTD_FLDR::cSend_HUNEtBB3_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cReceive_Cl4lHlXR_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cBinop_nQYN7UTO_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_x0rXjgh0, 0, m, &cVar_x0rXjgh0_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_h4i2XD7l_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 50.0f);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_DIVIDE, 100.0f, 0, m, &cBinop_nQYN7UTO_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_z3OOyjHA_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setElementToFrom(m, 0, n, 0);
  msg_setFloat(m, 1, 20.0f);
  sLine_onMessage(_c, &Context(_c)->sLine_EapmnhjA, 0, m, NULL);
}

void Heavy_WSTD_FLDR::cMsg_vdxQ66Ft_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 1.0f);
  cVar_onMessage(_c, &Context(_c)->cVar_ShFnw41F, 0, m, &cVar_ShFnw41F_sendMessage);
  cVar_onMessage(_c, &Context(_c)->cVar_S0dzBDYR, 0, m, &cVar_S0dzBDYR_sendMessage);
}

void Heavy_WSTD_FLDR::cReceive_Cl4lHlXR_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MIN, 1.0f, 0, m, &cBinop_0L9howgm_sendMessage);
}

void Heavy_WSTD_FLDR::cReceive_KRkbElfJ_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_DIVIDE, 100.0f, 0, m, &cBinop_nQYN7UTO_sendMessage);
}

void Heavy_WSTD_FLDR::cReceive_ihp3waXU_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_eoUG4Us9_sendMessage);
}

void Heavy_WSTD_FLDR::cReceive_LCQlUSXJ_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_S0dzBDYR, 0, m, &cVar_S0dzBDYR_sendMessage);
}

void Heavy_WSTD_FLDR::cReceive_GQRxiRAj_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_ShFnw41F, 0, m, &cVar_ShFnw41F_sendMessage);
}




/*
 * Context Process Implementation
 */

int Heavy_WSTD_FLDR::process(float **inputBuffers, float **outputBuffers, int n) {
  while (hLp_hasData(&inQueue)) {
    hv_uint32_t numBytes = 0;
    ReceiverMessagePair *p = reinterpret_cast<ReceiverMessagePair *>(hLp_getReadBuffer(&inQueue, &numBytes));
    hv_assert(numBytes >= sizeof(ReceiverMessagePair));
    scheduleMessageForReceiver(p->receiverHash, &p->msg);
    hLp_consume(&inQueue);
  }

  sendBangToReceiver(0xDD21C0EB); // send to __hv_bang~ on next cycle
  const int n4 = n & ~HV_N_SIMD_MASK; // ensure that the block size is a multiple of HV_N_SIMD

  // temporary signal vars
  hv_bufferf_t Bf0, Bf1, Bf2, Bf3;

  // input and output vars
  hv_bufferf_t O0, O1;
  hv_bufferf_t I0, I1;

  // declare and init the zero buffer
  hv_bufferf_t ZERO; __hv_zero_f(VOf(ZERO));

  hv_uint32_t nextBlock = blockStartTimestamp;
  for (int n = 0; n < n4; n += HV_N_SIMD) {

    // process all of the messages for this block
    nextBlock += HV_N_SIMD;
    while (mq_hasMessageBefore(&mq, nextBlock)) {
      MessageNode *const node = mq_peek(&mq);
      node->sendMessage(this, node->let, node->m);
      mq_pop(&mq);
    }

    // load input buffers
    __hv_load_f(inputBuffers[0]+n, VOf(I0));
    __hv_load_f(inputBuffers[1]+n, VOf(I1));

    // zero output buffers
    __hv_zero_f(VOf(O0));
    __hv_zero_f(VOf(O1));

    // process all signal functions
    __hv_varwrite_f(&sVarf_H1kktSyo, VIf(I0));
    __hv_line_f(&sLine_EapmnhjA, VOf(Bf0));
    __hv_var_k_f(VOf(Bf1), 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f);
    __hv_fma_f(VIf(I0), VIf(Bf0), VIf(Bf1), VOf(Bf1));
    __hv_var_k_f(VOf(Bf2), 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f);
    __hv_div_f(VIf(Bf1), VIf(Bf2), VOf(Bf2));
    __hv_floor_f(VIf(Bf2), VOf(Bf1));
    __hv_sub_f(VIf(Bf2), VIf(Bf1), VOf(Bf1));
    __hv_var_k_f(VOf(Bf2), 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f);
    __hv_sub_f(VIf(Bf1), VIf(Bf2), VOf(Bf2));
    __hv_abs_f(VIf(Bf2), VOf(Bf2));
    __hv_var_k_f(VOf(Bf1), 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f);
    __hv_var_k_f(VOf(Bf3), 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f);
    __hv_fms_f(VIf(Bf2), VIf(Bf1), VIf(Bf3), VOf(Bf3));
    __hv_neg_f(VIf(Bf3), VOf(Bf3));
    sEnv_process(this, &sEnv_I3YEyvOa, VIf(Bf3), &sEnv_I3YEyvOa_sendMessage);
    sEnv_process(this, &sEnv_vY4kIDob, VIf(I0), &sEnv_vY4kIDob_sendMessage);
    __hv_varread_f(&sVarf_tFRdCFny, VOf(Bf1));
    __hv_var_k_f(VOf(Bf2), 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f);
    __hv_sub_f(VIf(Bf1), VIf(Bf2), VOf(Bf2));
    __hv_var_k_f(VOf(Bf1), 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f);
    __hv_mul_f(VIf(Bf2), VIf(Bf1), VOf(Bf1));
    __hv_exp_f(VIf(Bf1), VOf(Bf1));
    __hv_mul_f(VIf(Bf3), VIf(Bf1), VOf(Bf1));
    __hv_varwrite_f(&sVarf_3yolUEdJ, VIf(Bf1));
    __hv_varwrite_f(&sVarf_sxxkYU0s, VIf(I1));
    __hv_var_k_f(VOf(Bf1), 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f);
    __hv_fma_f(VIf(I1), VIf(Bf0), VIf(Bf1), VOf(Bf1));
    __hv_var_k_f(VOf(Bf0), 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f);
    __hv_div_f(VIf(Bf1), VIf(Bf0), VOf(Bf0));
    __hv_floor_f(VIf(Bf0), VOf(Bf1));
    __hv_sub_f(VIf(Bf0), VIf(Bf1), VOf(Bf1));
    __hv_var_k_f(VOf(Bf0), 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f);
    __hv_sub_f(VIf(Bf1), VIf(Bf0), VOf(Bf0));
    __hv_abs_f(VIf(Bf0), VOf(Bf0));
    __hv_var_k_f(VOf(Bf1), 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f);
    __hv_var_k_f(VOf(Bf3), 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f);
    __hv_fms_f(VIf(Bf0), VIf(Bf1), VIf(Bf3), VOf(Bf3));
    __hv_neg_f(VIf(Bf3), VOf(Bf3));
    sEnv_process(this, &sEnv_HZ3RTB6V, VIf(Bf3), &sEnv_HZ3RTB6V_sendMessage);
    sEnv_process(this, &sEnv_OAnIN3rw, VIf(I1), &sEnv_OAnIN3rw_sendMessage);
    __hv_varread_f(&sVarf_08mScSWn, VOf(Bf1));
    __hv_var_k_f(VOf(Bf0), 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f);
    __hv_sub_f(VIf(Bf1), VIf(Bf0), VOf(Bf0));
    __hv_var_k_f(VOf(Bf1), 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f);
    __hv_mul_f(VIf(Bf0), VIf(Bf1), VOf(Bf1));
    __hv_exp_f(VIf(Bf1), VOf(Bf1));
    __hv_mul_f(VIf(Bf3), VIf(Bf1), VOf(Bf1));
    __hv_varwrite_f(&sVarf_UqUr6y8w, VIf(Bf1));
    __hv_varread_f(&sVarf_sxxkYU0s, VOf(Bf1));
    __hv_line_f(&sLine_9izQKvwX, VOf(Bf3));
    __hv_varread_f(&sVarf_UqUr6y8w, VOf(Bf0));
    __hv_line_f(&sLine_TNcFXvPg, VOf(Bf2));
    __hv_mul_f(VIf(Bf0), VIf(Bf2), VOf(Bf0));
    __hv_fma_f(VIf(Bf1), VIf(Bf3), VIf(Bf0), VOf(Bf0));
    __hv_add_f(VIf(Bf0), VIf(O1), VOf(O1));
    __hv_varread_f(&sVarf_H1kktSyo, VOf(Bf0));
    __hv_varread_f(&sVarf_3yolUEdJ, VOf(Bf1));
    __hv_mul_f(VIf(Bf1), VIf(Bf2), VOf(Bf2));
    __hv_fma_f(VIf(Bf0), VIf(Bf3), VIf(Bf2), VOf(Bf2));
    __hv_add_f(VIf(Bf2), VIf(O0), VOf(O0));

    // save output vars to output buffer
    __hv_store_f(outputBuffers[0]+n, VIf(O0));
    __hv_store_f(outputBuffers[1]+n, VIf(O1));
  }

  blockStartTimestamp = nextBlock;

  return n4; // return the number of frames processed

}

int Heavy_WSTD_FLDR::processInline(float *inputBuffers, float *outputBuffers, int n4) {
  hv_assert(!(n4 & HV_N_SIMD_MASK)); // ensure that n4 is a multiple of HV_N_SIMD

  // define the heavy input buffer for 2 channel(s)
  float **const bIn = reinterpret_cast<float **>(hv_alloca(2*sizeof(float *)));
  bIn[0] = inputBuffers+(0*n4);
  bIn[1] = inputBuffers+(1*n4);

  // define the heavy output buffer for 2 channel(s)
  float **const bOut = reinterpret_cast<float **>(hv_alloca(2*sizeof(float *)));
  bOut[0] = outputBuffers+(0*n4);
  bOut[1] = outputBuffers+(1*n4);

  int n = process(bIn, bOut, n4);
  return n;
}

int Heavy_WSTD_FLDR::processInlineInterleaved(float *inputBuffers, float *outputBuffers, int n4) {
  hv_assert(n4 & ~HV_N_SIMD_MASK); // ensure that n4 is a multiple of HV_N_SIMD

  // define the heavy input buffer for 2 channel(s), uninterleave
  float *const bIn = reinterpret_cast<float *>(hv_alloca(2*n4*sizeof(float)));
  #if HV_SIMD_SSE || HV_SIMD_AVX
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    __m128 a = _mm_load_ps(inputBuffers+i);                // LRLR
    __m128 b = _mm_load_ps(inputBuffers+4+i);              // LRLR
    __m128 x = _mm_shuffle_ps(a, b, _MM_SHUFFLE(2,0,2,0)); // LLLL
    __m128 y = _mm_shuffle_ps(a, b, _MM_SHUFFLE(3,1,3,1)); // RRRR
    _mm_store_ps(bIn+j, x);
    _mm_store_ps(bIn+n4+j, y);
  }
  #elif HV_SIMD_NEON
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    float32x4x2_t a = vld2q_f32(inputBuffers+i); // load and uninterleave
    vst1q_f32(bIn+j, a.val[0]);
    vst1q_f32(bIn+n4+j, a.val[1]);
  }
  #else // HV_SIMD_NONE
  for (int j = 0; j < n4; ++j) {
    bIn[0*n4+j] = inputBuffers[0+2*j];
    bIn[1*n4+j] = inputBuffers[1+2*j];
  }
  #endif

  // define the heavy output buffer for 2 channel(s)
  float *const bOut = reinterpret_cast<float *>(hv_alloca(2*n4*sizeof(float)));

  int n = processInline(bIn, bOut, n4);

  // interleave the heavy output into the output buffer
  #if HV_SIMD_AVX
  for (int i = 0, j = 0; j < n4; j += 8, i += 16) {
    __m256 x = _mm256_load_ps(bOut+j);    // LLLLLLLL
    __m256 y = _mm256_load_ps(bOut+n4+j); // RRRRRRRR
    __m256 a = _mm256_unpacklo_ps(x, y);  // LRLRLRLR
    __m256 b = _mm256_unpackhi_ps(x, y);  // LRLRLRLR
    _mm256_store_ps(outputBuffers+i, a);
    _mm256_store_ps(outputBuffers+8+i, b);
  }
  #elif HV_SIMD_SSE
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    __m128 x = _mm_load_ps(bOut+j);    // LLLL
    __m128 y = _mm_load_ps(bOut+n4+j); // RRRR
    __m128 a = _mm_unpacklo_ps(x, y);  // LRLR
    __m128 b = _mm_unpackhi_ps(x, y);  // LRLR
    _mm_store_ps(outputBuffers+i, a);
    _mm_store_ps(outputBuffers+4+i, b);
  }
  #elif HV_SIMD_NEON
  // https://community.arm.com/groups/processors/blog/2012/03/13/coding-for-neon--part-5-rearranging-vectors
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    float32x4_t x = vld1q_f32(bOut+j);
    float32x4_t y = vld1q_f32(bOut+n4+j);
    float32x4x2_t z = {x, y};
    vst2q_f32(outputBuffers+i, z); // interleave and store
  }
  #else // HV_SIMD_NONE
  for (int i = 0; i < 2; ++i) {
    for (int j = 0; j < n4; ++j) {
      outputBuffers[i+2*j] = bOut[i*n4+j];
    }
  }
  #endif

  return n;
}
