/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_FLDR_HPP_
#define _HEAVY_CONTEXT_WSTD_FLDR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvControlIf.h"
#include "HvControlCast.h"
#include "HvSignalLine.h"
#include "HvSignalEnvelope.h"
#include "HvSignalVar.h"
#include "HvMath.h"
#include "HvControlVar.h"
#include "HvControlBinop.h"

class Heavy_WSTD_FLDR : public HeavyContext {

 public:
  Heavy_WSTD_FLDR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_FLDR();

  const char *getName() override { return "WSTD_FLDR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        FLDR = 0xE42C892E, // Fldr
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_0L9howgm_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_w17KrEW1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_esfZ05Jc_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_oHDU39sz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_PtMAqfZJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_0uLefYhK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_o4ASLpzl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_IkpB58p1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_BMwxYGUV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_o2iN37Zd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_GlpWJDOc_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_x0rXjgh0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_eoUG4Us9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_ShFnw41F_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_kEC9XyTZ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_gmxU13Fs_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_j1IshWSn_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_I3YEyvOa_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_vY4kIDob_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_pMIzpqZH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_0U9xWbv8_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Ykokv1Q5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ronlnmAF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_W3YpdWjS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_kFE29kcD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_MQgTonZi_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_Uirb6scD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_o3pODnnw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_EtZB5pnc_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_rxxZNeFn_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_dgMDta5Z_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_RXW5loWX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_TCycJSp0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_SGyujIhK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_HZ3RTB6V_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_OAnIN3rw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_8XV3yZLt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_niQfnDWN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ILWuA3Kj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_GpdJq9Rv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_A7UxbEqk_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_H9qgHDE3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_eDdqYkOv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_iYeJ7Rmf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_fluVyqkI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_XtDeUwUx_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_0nAsFLBA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_7cgjUbNM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_S0dzBDYR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_tuWzjzdS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_5BqkfS9f_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_Oof5MhVd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_HUNEtBB3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_nQYN7UTO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_h4i2XD7l_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_z3OOyjHA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_vdxQ66Ft_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_Cl4lHlXR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_KRkbElfJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_ihp3waXU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_LCQlUSXJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_GQRxiRAj_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_EapmnhjA;
  SignalEnvelope sEnv_I3YEyvOa;
  SignalEnvelope sEnv_vY4kIDob;
  SignalEnvelope sEnv_HZ3RTB6V;
  SignalEnvelope sEnv_OAnIN3rw;
  SignalLine sLine_9izQKvwX;
  SignalLine sLine_TNcFXvPg;
  ControlBinop cBinop_0L9howgm;
  ControlBinop cBinop_w17KrEW1;
  ControlBinop cBinop_esfZ05Jc;
  ControlBinop cBinop_oHDU39sz;
  ControlBinop cBinop_o4ASLpzl;
  ControlBinop cBinop_BMwxYGUV;
  ControlBinop cBinop_o2iN37Zd;
  ControlBinop cBinop_GlpWJDOc;
  ControlVar cVar_x0rXjgh0;
  ControlVar cVar_ShFnw41F;
  SignalVarf sVarf_tFRdCFny;
  ControlVar cVar_kEC9XyTZ;
  ControlIf cIf_gmxU13Fs;
  ControlIf cIf_j1IshWSn;
  ControlBinop cBinop_pMIzpqZH;
  ControlBinop cBinop_Ykokv1Q5;
  ControlBinop cBinop_ronlnmAF;
  ControlBinop cBinop_MQgTonZi;
  ControlBinop cBinop_o3pODnnw;
  SignalVarf sVarf_08mScSWn;
  ControlVar cVar_RXW5loWX;
  ControlIf cIf_TCycJSp0;
  ControlIf cIf_SGyujIhK;
  ControlBinop cBinop_8XV3yZLt;
  ControlBinop cBinop_ILWuA3Kj;
  ControlBinop cBinop_GpdJq9Rv;
  ControlBinop cBinop_eDdqYkOv;
  ControlBinop cBinop_fluVyqkI;
  ControlVar cVar_S0dzBDYR;
  ControlBinop cBinop_tuWzjzdS;
  SignalVarf sVarf_H1kktSyo;
  SignalVarf sVarf_3yolUEdJ;
  SignalVarf sVarf_sxxkYU0s;
  SignalVarf sVarf_UqUr6y8w;
  ControlBinop cBinop_nQYN7UTO;
};

#endif // _HEAVY_CONTEXT_WSTD_FLDR_HPP_
